/***********************************************************************************
 * NAME	: DoublePorts.cpp
 * DESC	: Test double serial ports program - double loop
 * DATE	: 7/24/2002
 * PGMR	: Y. Bai
 **********************************************************************************/
#include "DoublePorts.h"
#include "Menu.h"

void main()
{
	int   index;
	bool  select = TRUE;
	char* userInput;
	char* comPort;
	ERR_CODE  ecRet = OK;
	unsigned char cPort;

	comPort   = new char[128];
	userInput = new char[128];
	pPortName portName = new PortName;

	TestPassed = FALSE;
	sPortFlg = FALSE;
	rPortFlg = FALSE;

	while(select)
	{
		printf("\n");
		for (index = 0; index < MAX_MENU; index++)
			printf(menuData[index]);

		printf(menuTitle[0]);	
		scanf("%s", userInput);
		userInput = strlwr(userInput);
		cPort = (unsigned char)userInput[0];
		getMenuItem(cPort, comPort);
		if (strcmp(comPort, "EXIT") == 0 || strcmp(comPort, "NULL") == 0)
			select = FALSE;
		else if (strcmp(comPort, "INIT") == 0)
		{
			ecRet = startPort(portName);
			if (ecRet != OK)
			{
				printf("ERROR in startPort\n");
				break;
			}
		}
		else if (strcmp(comPort, "TEST") == 0)
		{
			ecRet = TestPort(portName->sComPort, portName->rComPort, 1);
			if (ecRet)
			{
				printf("ERROR in TestPort\n");
				break;
			}
		}
		else if (strcmp(comPort, "KILL") == 0)
			printf("\n");
		if (TestPassed)
			printf("\n**** TEST PASSED ***\n");
	}
	delete  userInput;
	delete  comPort;
	delete  portName;

	return;
}

ERR_CODE startPort(pPortName pName)
{
	ERR_CODE  ecRet = OK;
	char* comPort;
	unsigned char sendComm = '1', recvComm = '2';
	unsigned char userInput[MAX_STRING];

	comPort   = new char[128];

	printf("Defaults Ports - send: COM %c  recv: COM %c\n", sendComm, recvComm);

	printf("\nChange Default Ports [N]?");
	scanf("%s", userInput);
	if (userInput[0] == 'y'||userInput[0] == 'Y')
	{
		printf("Enter send COM Port [%c]: ", sendComm);
		scanf("%s", userInput);
		if(userInput[0] != 0)
			sendComm = userInput[0];
		printf( "Enter recv COM Port [%c]: ", recvComm);
		scanf("%s", userInput);
		if(userInput[0] != 0)
			recvComm = userInput[0];
	}
	getMenuItem(sendComm, comPort);
	printf("select send port: %s\n", comPort);
	strcpy(pName->sComPort, comPort);
	ecRet = SetupPort(comPort, 1);
	if (ecRet)
	{
		delete comPort;
		return ecRet;
	}
	getMenuItem(recvComm, comPort);
	printf("select recv port: %s\n", comPort);
	strcpy(pName->rComPort, comPort);
	ecRet = SetupPort(comPort, 2);
	if (ecRet)
	{
		delete comPort;
		return ecRet;
	}

	delete comPort;

	return ecRet;
}

ERR_CODE SetupPort(char* cPort, int pFlg)
{
	ERR_CODE ecRet = OK;
	pSerialCreate  pParam;

	pParam = new SerialCreate;

	pParam->lBaudRate = BAUD_RATE;
	pParam->lDataBits = NUM_BITS;
	pParam->lTimeout  = TIME_OUT;

	if (sPortFlg && pFlg == 1)
		return ecRet;
	if (rPortFlg && pFlg == 2)
		return ecRet;

	ecRet = PortInitialize(cPort, pParam);
	if (ecRet != OK)
		printf("ERROR in PortInitialize()!\n");
	else
	{
		if (pFlg == 1)
		{
			sPortFlg = true;
			hsPort = pParam->hPort;
		}
		else
		{
			rPortFlg = true;
			hrPort = pParam->hPort;
		}
	}

	delete pParam;
	return ecRet;
}

ERR_CODE TestPort(char* sPort, char* rPort, BOOL display)
{
	BYTE   sByte;
	int    numByte = NUM_BYTE, MaxByte = MAX_BYTE;
	ERR_CODE ecRet = OK;
	CommPortClass* comPort = new CommPortClass;

	comPort->handlePort = hrPort;
	comPort->iMaxChars  = NUM_BYTE;

	for (sByte = START_BYTE; sByte <= MaxByte; sByte++)
	{
		ecRet = PortWrite(hsPort, sByte, numByte);
		if (ecRet)
		{
			printf("PortWrite() is failed\n");
			TestPassed = FALSE;
			CloseHandle(hsPort);
			return EC_WRITE_FAIL;
		}
		if (display)
			printf("%s Port Sending:  = %c\n", sPort, sByte);

		ecRet = PortRead(comPort);
		if (ecRet)
		{
			printf("PortRead() is failed\n");
			TestPassed = FALSE;
			CloseHandle(hrPort);
			return EC_READ_FAIL;
		}
		if (display)
			printf("%s Port Received: = %c\n", rPort, comPort->bByte);

		if (sByte != comPort->bByte)
		{
			TestPassed = FALSE;
			return EC_TEST_FAIL;
		}
	}
	TestPassed = TRUE;

	delete comPort;
	return ecRet;
}

int getMenuItem(unsigned char mPort, char* ret)
{
	switch (mPort)
	{
	case '1': strcpy(ret, "COM1");
		break;
	case '2': strcpy(ret, "COM2");;
		break;
	case '3': strcpy(ret, "COM3");
		break;
	case '4': strcpy(ret, "COM4");
		break;
	case '5': strcpy(ret, "COM5");
		break;
	case '6': strcpy(ret, "COM6");
		break;
	case '7': strcpy(ret, "COM7");
		break;
	case '8': strcpy(ret, "COM8");
		break;
	case 'a': strcpy(ret, "INIT");
		break;
	case 'b': strcpy(ret, "TEST");
		break;
	case 'c': strcpy(ret, "KILL");
		break;
	case 'x': strcpy(ret, "EXIT");
		break;
	default: printf("Invalid Selection\n");
		strcpy(ret, "NULL");
	}

	return 0;
}
