/***********************************************************************************
 * NAME	: SimpleCOM.cpp
 * DESC	: Simple serial port test program - single loop
 * DATE	: 7/15/2002
 * PGMR	: Y. Bai
 **********************************************************************************/
#include "SimpleCOM.h"
#include "Menu.h"

void main()
{
	int   index;
	bool  select = TRUE;
	char* userInput;
	char* comPort;
	ERR_CODE  ecRet = OK;
	unsigned char cPort;

	userInput = new char[128];
	comPort   = new char[128];

	TestPassed = FALSE;
	while(select)
	{
		printf("\n");
		for (index = 0; index < MAX_MENU; index++)
		{
			PortCreateflg[index] = FALSE;
			printf(menuData[index]);
		}

		printf(menuTitle[0]);
		scanf("%s", userInput);
		userInput = strlwr(userInput);
		cPort = (unsigned char)userInput[0];
		getMenuItem(cPort, comPort);
		printf("select = %s\n", comPort);
		if (strcmp(comPort, "EXIT") == 0 || strcmp(comPort, "NULL") == 0)
			select = FALSE;
		else
		{
			ecRet = SetupPort(comPort, (int)cPort);
			if (ecRet)
			{
				select = FALSE;
				break;
			}
			ecRet = TestPort(comPort, 1);
			if (ecRet)
			{
				select = FALSE;
				break;
			}
			if (TestPassed)
				printf("\n**** %s TEST PASSED ***\n", comPort);
		}
	}
	delete  userInput;
	delete  comPort;

	return;
}

ERR_CODE SetupPort(char* cPort, int nPort)
{
	ERR_CODE ecRet = OK;
	pSerialCreate  pParam;

	pParam = new SerialCreate;

	pParam->lBaudRate = BAUD_RATE;
	pParam->lDataBits = NUM_BITS;
	pParam->lTimeout  = TIME_OUT;

	if (PortCreateflg[nPort])
		return ecRet;

	ecRet = PortInitialize(cPort, pParam);
	if (ecRet != OK)
		printf("ERROR in PortInitialize()!\n");
	else
	{
		PortCreateflg[nPort] = true;
		hPort = pParam->h_Port;
	}

	delete pParam;

	return ecRet;
}

ERR_CODE TestPort(char* cPort, BOOL display)
{
	BYTE   sByte;
	int    numByte = NUM_BYTE, MaxByte = MAX_BYTE;
	ERR_CODE ecRet = OK;
	CommPortClass* comPort = new CommPortClass;

	comPort->handlePort = hPort;
	comPort->iMaxChars  = NUM_BYTE;

	for (sByte = START_BYTE; sByte <= MaxByte; sByte++)
	{
		ecRet = PortWrite(hPort, sByte, numByte);
		if (ecRet)
		{
			printf("PortWrite() is failed\n");
			TestPassed = FALSE;
			CloseHandle(hPort);
			return EC_WRITE_FAIL;
		}
		if (display)
			printf("%s Port Sending:  = %c\n", cPort, sByte);

		ecRet = PortRead(comPort);
		if (ecRet)
		{
			printf("PortRead() is failed\n");
			TestPassed = FALSE;
			CloseHandle(hPort);
			return EC_READ_FAIL;
		}
		if (display)
			printf("%s Port Received: = %c\n", cPort, comPort->bByte);

		if (sByte != comPort->bByte)
		{
			TestPassed = FALSE;
			return EC_TEST_FAIL;
		}
	}
	TestPassed = TRUE;

	delete comPort;

	return ecRet;
}

int getMenuItem(unsigned char mPort, char* ret)
{
	switch (mPort)
	{
	case 'a': strcpy(ret, "COM1");
		break;
	case 'b': strcpy(ret, "COM2");;
		break;
	case 'c': strcpy(ret, "COM3");
		break;
	case 'd': strcpy(ret, "COM4");
		break;
	case 'e': strcpy(ret, "COM5");
		break;
	case 'f': strcpy(ret, "COM6");
		break;
	case 'g': strcpy(ret, "COM7");
		break;
	case 'h': strcpy(ret, "COM8");
		break;
	case 'x': strcpy(ret, "EXIT");
		break;
	default: printf("Invalid Selection\n");
		strcpy(ret, "NULL");
	}

	return 0;
}
