/************************************************************************************
 * NAME		: SerialDLL.cpp
 * DESC.	: DLL functions to interface to RS-232 serial port, called by VB.
 * PRGMER.	: Y. Bai
 * DATE		: 7/2/2003
 ************************************************************************************/

#include <stdio.h>
#include <windows.h>
#include <stdlib.h>
#include <string.h>
#include "SerialDLL.h"

DllExport int CALLBACK Setup(int cPort, int bRate)
{
	ERR_CODE  rc = OK;
	pSerialCreate  pParam;

	pParam = new SerialCreate;

	pParam->lBaudRate = bRate;
	pParam->lDataBits = 8;
	pParam->lTimeout  = 3000;
	
	switch(cPort)
	{
	case 1:
		sPortName = "COM1";
		break;
	case 2:
		sPortName = "COM2";
		break;
	case 3:
		sPortName = "COM3";
		break;
	case 4:
		sPortName = "COM4";
		break;
	default:
		return EC_INVALIDPORT;
	}

	if (PortCreateflg)
	{
		msg("Port has been Setup ");
		return rc;
	}
	rc = PortInitialize(sPortName, pParam);
	if (rc != 0)
		msg("ERROR in PortInitialize()!");
	
	delete pParam;
	PortCreateflg = true;

	return rc;
}

DllExport int CALLBACK Write(int sdata)
{
	int  numByte = 1;
	char sByte[8];
	ERR_CODE rc = OK;
	
	sprintf(sByte, "%c", sdata);
	rc = PortWrite(sByte, numByte);
	if (rc != 0)
		msg("ERROR in PortWrite() !");
	
	return rc;
}

DllExport int CALLBACK Read()
{
	int  idata;
	char cdata[8];
	ERR_CODE rc = OK;
	CommPortClass* commClass;

	commClass = new CommPortClass;
	commClass->pcBuffer = new char;
	commClass->iMaxChars = 1;
	
	rc = PortRead(commClass);
	if (rc != 0)
		msg("ERROR in PortRead()! ");

	sprintf(cdata, "%d", commClass->pcBuffer[0]);
	idata = atoi(cdata);
	delete commClass->pcBuffer;
	delete commClass;

	return idata;
}

DllExport int CALLBACK Close()
{
	ERR_CODE rc = OK;

	if (PortCreateflg)
		CloseHandle(hPort);
	PortCreateflg = false;
	
	return rc;
}

ERR_CODE PortInitialize(LPTSTR lpszPortName, pSerialCreate pCreate)
{
	  DWORD			 dwError;
	  DCB			 PortDCB;  
	  ERR_CODE       ecStatus = OK;
	  COMMTIMEOUTS   CommTimeouts;
	  unsigned char  dBit;

	  // check if the port has been created...
	  if (PortCreateflg)
	  {
		  msg("Port has been initialized!");
		  return ecStatus;
	  }
	  // Open the serial port.
	  hPort = CreateFile(lpszPortName,  // Pointer to the name of the port
						 GENERIC_READ | GENERIC_WRITE,
										// Access (read/write) mode
						 0,             // Share mode
						 NULL,          // Pointer to the security attribute
						 OPEN_EXISTING, // How to open the serial port
						 0,             // Port attributes
						 NULL);         // Handle to port with attribute to copy										

	  // If it fails to open the port, return error.
	  if ( hPort == INVALID_HANDLE_VALUE ) 
	  {
		// Could not open the port.
		dwError = GetLastError();
		msg("Unable to open the port");
		CloseHandle(hPort);
		return EC_FOPEN;
	  }
	  
	  PortCreateflg = TRUE;
	  PortDCB.DCBlength = sizeof(DCB);     

	  // Get the default port setting information.
	  GetCommState(hPort, &PortDCB);
	  
	  // Change the DCB structure settings.
	  PortDCB.BaudRate = pCreate->lBaudRate;    // Current baud 
	  PortDCB.fBinary  = TRUE;					// Binary mode; no EOF check 
	  PortDCB.fParity  = TRUE;					// Enable parity checking. 
	  PortDCB.fOutxCtsFlow = FALSE;				// No CTS output flow control 
	  PortDCB.fOutxDsrFlow = FALSE;				// No DSR output flow control 
	  PortDCB.fDtrControl = DTR_CONTROL_ENABLE; // DTR_CONTROL_DISABLE
												// DTR flow control type 
	  PortDCB.fDsrSensitivity = FALSE;			// DSR sensitivity 
	  PortDCB.fTXContinueOnXoff = TRUE;			// FALSE	// XOFF continues Tx 
	  PortDCB.fOutX = FALSE;					// No XON/XOFF out flow control 
	  PortDCB.fInX  = FALSE;					// No XON/XOFF in flow control 
	  PortDCB.fErrorChar = FALSE;				// Disable error replacement. 
	  PortDCB.fNull = FALSE;					// Disable null stripping. 
	  PortDCB.fRtsControl = RTS_CONTROL_ENABLE; // RTS_CONTROL_DISABLE
												// RTS flow control 
	  PortDCB.fAbortOnError = FALSE;			// Do not abort reads/writes on error.
	  dBit = (unsigned char)pCreate->lDataBits;	
	  PortDCB.ByteSize = dBit;				    // Number of bits/bytes, 4-8 
	  PortDCB.Parity = NOPARITY;				// 0-4=no,odd,even,mark,space 
	  PortDCB.StopBits = ONESTOPBIT;			// 0,1,2 = 1, 1.5, 2 

	  // Configure the port according to the specifications of the DCB structure.
	  if (!SetCommState (hPort, &PortDCB))
	  {
		// Could not create the read thread.
		dwError = GetLastError();
		msg("Unable to configure the serial port");
		return EC_INVAL_CONFIG;
	  }

	  // Retrieve the time-out parameters for all read and write operations on the port. 
	  GetCommTimeouts(hPort, &CommTimeouts);

	  // Change the COMMTIMEOUTS structure settings.
	  CommTimeouts.ReadIntervalTimeout = MAXDWORD;  
	  CommTimeouts.ReadTotalTimeoutMultiplier = 0;  
	  CommTimeouts.ReadTotalTimeoutConstant = 0;    
	  CommTimeouts.WriteTotalTimeoutMultiplier = 10;  
	  CommTimeouts.WriteTotalTimeoutConstant = 1000;    

	  // Set the time-out parameters for all read and write operations on the port. 
	  if (!SetCommTimeouts (hPort, &CommTimeouts))
	  {
		// Could not create the read thread.
		dwError = GetLastError();
		msg("Unable to set the time-out parameters");
		return EC_TIMEOUT_SET;
	  }

	  EscapeCommFunction(hPort, SETDTR);
	  EscapeCommFunction(hPort, SETRTS);

	  return ecStatus;
}

/********************************************************************************
 * NAME		: PortWrite()
 * DESC.	: Write data to the port
 * DATE		: 11/9/00
 * PRGMER.	: Y. Bai
 ********************************************************************************/
ERR_CODE PortWrite(char* bByte, int NumByte)
{
	  DWORD dwError;
      DWORD dwNumBytesWritten;
	  ERR_CODE ecStatus = OK;

	  if (!WriteFile (hPort,				// Port handle
					  bByte,				// Pointer to the data to write 
					  NumByte,				// Number of bytes to write
					  &dwNumBytesWritten,	// Pointer to the number of bytes written
					  NULL))				// Must be NULL for Windows CE
	  {
		// WriteFile failed. Report error.
		dwError = GetLastError ();
		msg("ERROR in PortWrite ..");
		return EC_WRITE_FAIL;
	  }

	  return ecStatus;
}

/**********************************************************************************
 * NAME		: PortRead()
 * DESC.	: Read data from serial port
 * PGRMER.	: Y. Bai
 * DATE		: 11/10/00
 **********************************************************************************/
ERR_CODE PortRead(CommPortClass *hCommPort)
{  
	  HANDLE hThread;			// handler for port read thread
	  DWORD  IDThread;
	  DWORD  Ret, ExitCode;
	  DWORD	 dTimeout = 5000;	// define time out value: 5 sec.
	  ERR_CODE ecStatus = OK;

	  if (!(hThread = CreateThread(NULL,			// no security attributes
				    0,					            // use default stack size
				    (LPTHREAD_START_ROUTINE) ThreadFunc,
				    (LPVOID)hCommPort,              // parameter to thread funciton
				    CREATE_SUSPENDED,	            // creation flag - suspended
				    &IDThread) ) )					// returns thread ID
        {
			msg("Create Read Thread failed");
	 		return EC_CREATE_THREAD;
	    }

	  ResumeThread(hThread);		// start thread now
	  
	  Ret = WaitForSingleObject(hThread, dTimeout);
	  if (Ret == WAIT_OBJECT_0)
	  {
		  // data received & process it...
		  // Need to do nothing because the data has been stored in the hCommPort in Thread Func.
		  // close thread handle
		  CloseHandle(hThread);
	  }
	  else if (Ret == WAIT_TIMEOUT)
	  {
		  // time out happened, warning & kill thread
		  Ret = GetExitCodeThread(hThread, &ExitCode);
		  msg("Time out happened in PortRead() ");
		  if (ExitCode == STILL_ACTIVE)
		  {
			  TerminateThread(hThread, ExitCode);
			  CloseHandle(hThread);
			  return EC_RECV_TIMEOUT;
		  }
		  else
		  {
			  CloseHandle(hThread);
			  msg("ERROR in GetExitCodeThread: != STILL_ACTIVE ");
			  ecStatus = EC_EXIT_CODE;
		  }
	  }
	  else
	  {
		  msg("ERROR in WaitFor SingleObject ");
		  ecStatus = EC_WAIT_SINGLEOBJ;
	  }
	  
	  return ecStatus;
}
	
/***************************************************************************************
* NAME		: ThreadFunc()
* DESC.		: This funciton starts to read the port data.
*			: The reason to use this thread is to overcome the hung up introduced by the
*			: WaitCommEvent() function which waits for only the comm. event, not synchro-
*			: nition event. The program will be dead-cycle without this thread.
* PGRMER.	: Y. Bai
* DATE		: 12/4/00
****************************************************************************************/
void WINAPI ThreadFunc(void* hCommPorts)
{
	  char  Byte;
	  BOOL  bResult, fDone;
	  int	nTotRead = 0;
 	  DWORD dwCommModemStatus, dwBytesTransferred;  
	  CommPortClass*  CommPorts;
	  ERR_CODE ecStatus = OK;

	  CommPorts = (CommPortClass* )hCommPorts;	
	
 	  // Specify a set of events to be monitored for the port.
 	  SetCommMask(hPort, EV_RXCHAR | EV_CTS | EV_DSR | EV_RLSD | EV_RING);

	  fDone = FALSE;
 	  while (!fDone) 
 	  {
		// Wait for an event to occur for the port.
 		WaitCommEvent(hPort, &dwCommModemStatus, 0);

 		// Re-specify the set of events to be monitored for the port.
 		SetCommMask(hPort, EV_RXCHAR | EV_CTS | EV_DSR |EV_RLSD| EV_RING);
 		if (dwCommModemStatus & EV_RXCHAR||dwCommModemStatus & EV_RLSD)	// received the char_event
 		{
 		  // Loop for waiting for the data.
 		  do 
 		  {
 			// Read the data from the serial port.
 			bResult = ReadFile(hPort, &Byte, 1, &dwBytesTransferred, 0);
 
 			if (!bResult)
			{				
				msg("ERROR in ReadFile !");
				fDone = TRUE;
				break;
			}
			else
			{
            	// Display the data read.
 				if (dwBytesTransferred == 1)
				{
					if (Byte == 0x0D ||Byte == 0x0A)	// null char or LF
					{
						CommPorts->piNumRcvd = nTotRead;
						fDone = TRUE;
						break;
					}
					CommPorts->pcBuffer[nTotRead] = Byte;	
					nTotRead++;
					if (nTotRead == CommPorts->iMaxChars)
					{
						fDone = TRUE;
						break;
					}
				}
				else
				{
					if (Byte == 0x0D ||Byte == 0x0A)	// null char
					{						
						msg("Received null character ");
						fDone = TRUE;
						break;
					}
				}
			}

		  }while (dwBytesTransferred == 1); //while (nTotRead < pRecv->iMaxChars); 

		}
 	  }		// while

 	  return;
}

