/***************************************************************************************
 * NAME:	CINReadData.cpp
 * DESC:	Interface program to MAX187 Serial A/D converter
 * PGMR:	Y. Bai
 * DATE:	11/6/2003
 **************************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "CINReadData.h"

MgErr  CINProperties(int32 prop, void *data)
{
	switch (prop)
	{
	case kCINIsReentrant:
		*(Bool32 *)data = TRUE;
		return noErr;
	}
	return mgNotSupported;
}

MgErr CINRun(int16 *DataOutput, int32 *hPortIn, int32 *hPortOut, LVBoolean *flg)
{
	/* Insert code here */
	ERR_CODE rc = OK;
	short ad_data;
	CommPortClass* commClass;
	hPort = (HANDLE)(*hPortIn);
	*hPortOut = *hPortIn;

	commClass = new CommPortClass;
	commClass->iMaxChars = MAX_BYTE;
	commClass->iMaxBits  = MAX_BITS;
	
	EscapeCommFunction(hPort, SETDTR);	//Set DTR to disable the SCLK
	EscapeCommFunction(hPort, CLRRTS);	//Reset RTS to disable the CS-bar

	rc = PortRead(commClass);
	if (rc != 0)
		*flg = LVTRUE;
	else
		*flg = LVFALSE;

	ad_data = commClass->pcBuffer;
	delete commClass;

	*DataOutput = ad_data;

	return noErr;
}
/**********************************************************************************
 * NAME		: PortRead()
 * DESC.	: Read data from serial port
 * PGRMER.	: Y. Bai
 * DATE		: 10/31/03
 **********************************************************************************/
ERR_CODE PortRead(CommPortClass *hCommPort)
{  
	HANDLE hThread;			// handler for port read thread
	DWORD  IDThread;
	DWORD  Ret, ExitCode;
	DWORD  dTimeout = 500;	// define time out value: 500 ms.
	ERR_CODE ecStatus = OK;

	if (!(hThread = CreateThread(NULL,			    // no security attributes
				    0,					            // use default stack size
				    (LPTHREAD_START_ROUTINE) ThreadFunc,
				    (LPVOID)hCommPort,              // parameter to thread function
				    CREATE_SUSPENDED,	            // creation flag - suspended
				    &IDThread) ) )					// returns thread ID
    {
		msg("Create Read Thread failed");
	 	return EC_CREATE_THREAD;
	}

	  ResumeThread(hThread);		// start thread now

	  Ret = WaitForSingleObject(hThread, dTimeout);
	  if (Ret == WAIT_OBJECT_0)
	  {
		  // data received & process it...need to do nothing because the data 
		  // has been stored in the hCommPort in Thread Func. Close thread handle
		  CloseHandle(hThread);
	  }
	  else if (Ret == WAIT_TIMEOUT)
	  {
		  // time out happened, warning & kill thread
		  Ret = GetExitCodeThread(hThread, &ExitCode);
		  if (ExitCode == STILL_ACTIVE)
		  {
			  TerminateThread(hThread, ExitCode);
			  CloseHandle(hThread);
			  return EC_RECV_TIMEOUT;
		  }
		  else
		  {
			  CloseHandle(hThread);
			  msg("ERROR in GetExitCodeThread: != STILL_ACTIVE ");
			  ecStatus = EC_EXIT_CODE;
		  }
	  }
	  else
	  {
		  msg("ERROR in WaitFor SingleObject ");
		  ecStatus = EC_WAIT_SINGLEOBJ;
	  }
	  
	  return ecStatus;
}
/***************************************************************************************
* NAME		: ThreadFunc()
* DESC.		: This funciton starts to read the data from the port.
*			: The reason to use this thread is to overcome the hung up introduced by the
*			: WaitCommEvent() function which waits for only the comm. event, not synchro-
*			: nition event. The program will be dead-cycle without this thread.
* PGRMER.	: Y. Bai
* DATE		: 12/4/03
****************************************************************************************/
void WINAPI ThreadFunc(void* hCommPorts)
{
	BOOL   fDone;
	short  ad_data = 0;
	DWORD  dwModemStatus;
	int    index, nTotRead = 0;
 	DWORD  dwCommModemStatus;  
	CommPortClass*  CommPorts;
	ERR_CODE ecStatus = OK;

	CommPorts = (CommPortClass* )hCommPorts;	

	EscapeCommFunction(hPort, SETRTS);		// Set RTS to enable the CS-bar to start A/D 
	// Specify a set of events to be monitored for the port.
 	SetCommMask(hPort, EV_DSR);
	fDone = FALSE;
 	while (!fDone) 
 	{
		// Wait for an event to occur for the port.
 		WaitCommEvent(hPort, &dwCommModemStatus, 0);
 		if (dwCommModemStatus & EV_DSR)	// received the DSR event
 		{
 		    for (index = 0; index < CommPorts->iMaxBits; index++)
			{
				EscapeCommFunction(hPort, CLRDTR);	//Reset DTR (provide HIGH in SCLK) 
				Delay(10);
				EscapeCommFunction(hPort, SETDTR);	//Set DTR (provide LOW in SCLK) 
				Delay(10);		
				if (!GetCommModemStatus(hPort, &dwModemStatus))
				{
					msg("ERROR in GetModemStatus()!");
					fDone = TRUE;
					break;
				}
				if (MS_DSR_ON & dwModemStatus)		//DSR status is HIGH.
					ad_data = ad_data + 0;
				else								//DSR status is LOW
					ad_data++;
				ad_data = ad_data << 1;
			}
			EscapeCommFunction(hPort, SETDTR);	//Set DTR (provide LOW in SCLK) 
			EscapeCommFunction(hPort, CLRRTS);	//Reset CS to deselect A/D 
			ad_data = ad_data >> 1;
			CommPorts->pcBuffer = ad_data;
			fDone = TRUE;
			break;
		}
	}
}
void Delay(int num)
{
	int l, m, n = 100*num;

	for (m = 0; m < n; m++)
		l++;
}
