/***************************************************************************************
 * NAME:	CINSetup.cpp
 * DESC:	Interface program to MAX187 Serial A/D converter
 * PGMR:	Y. Bai
 * DATE:	11/6/2003
 **************************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "CINSetup.h"

MgErr CINRun(TD1Hdl PortSettings, int32 *PortHandle, int32 *flg)
{
	/* Insert code here */
    ERR_CODE  rc = OK;              //COM[0] = port number, COM[1] = baud_rate,
	pSerialCreate  pParam;			//COM[2] = data_bits,   COM[3] = stop_bit
	pParam = new SerialCreate;		//COM[4] = parity,      COM[5] = timeout_value

	pParam->lBaudRate = (*PortSettings)->COM[1];
	pParam->lDataBits = (*PortSettings)->COM[2];
	pParam->lTimeout  = (*PortSettings)->COM[5];

	switch((int)(*PortSettings)->COM[0])
	{
	case 1:
		sPortName = "COM1";
		break;
	case 2:
		sPortName = "COM2";
		break;
	case 3:
		sPortName = "COM3";
		break;
	case 4:
		sPortName = "COM4";
		break;
	default:
		*flg = EC_INVALIDPORT;
	}
	rc = PortInitialize(sPortName, pParam);
	if (rc != 0)
		msg("ERROR in PortInitialize()!");
	
	delete pParam;
	*flg = 0;
	*PortHandle = (long)hPort;

	return noErr;
}

ERR_CODE PortInitialize(char* lpszPortName, pSerialCreate pCreate)
{
	DWORD  dwError;
	DCB	   PortDCB;  
	ERR_CODE       ecStatus = OK;
	COMMTIMEOUTS   CommTimeouts;
	UCHAR  dBit;

	// Open the serial port.
	hPort = CreateFile(lpszPortName,  // Pointer to the name of the port
					   GENERIC_READ | GENERIC_WRITE, // Access (read/write) mode
					   0,             // Share mode
					   NULL,          // Pointer to the security attribute
					   OPEN_EXISTING, // How to open the serial port
					   0,             // Port attributes
					   NULL);         // Handle to port with attribute to copy										

	// If it fails to open the port, return error.
	if ( hPort == INVALID_HANDLE_VALUE ) 
	{
		// Could not open the port.
		dwError = GetLastError();
		msg("Unable to open the port");
		CloseHandle(hPort);
		return EC_FOPEN;
	}
	PortDCB.DCBlength = sizeof(DCB);     

	// Get the default port setting information.
	GetCommState(hPort, &PortDCB);
	  
	// Change the DCB structure settings.
	PortDCB.BaudRate = pCreate->lBaudRate;    // Current baud 
	PortDCB.fBinary  = TRUE;				  // Binary mode; no EOF check 
	PortDCB.fParity  = FALSE;				  // Enable parity checking. 
	PortDCB.fOutxCtsFlow = FALSE;			  // No CTS output flow control 
	PortDCB.fOutxDsrFlow = FALSE;			  // No DSR output flow control 
	PortDCB.fDtrControl = DTR_CONTROL_ENABLE; // DTR flow control type 
	PortDCB.fDsrSensitivity = FALSE;		  // DSR sensitivity 
	PortDCB.fTXContinueOnXoff = FALSE;		  // FALSE	// XOFF continues Tx 
	PortDCB.fOutX = FALSE;					  // No XON/XOFF out flow control 
	PortDCB.fInX  = FALSE;					  // No XON/XOFF in flow control 
	PortDCB.fErrorChar = FALSE;				  // Disable error replacement. 
	PortDCB.fNull = FALSE;					  // Disable null stripping. 
	PortDCB.fRtsControl = RTS_CONTROL_ENABLE; // RTS flow control
	PortDCB.fAbortOnError = FALSE;			  // Do not abort reads/writes on error.
	dBit = (UCHAR)pCreate->lDataBits;		  // Assign data bits	
	PortDCB.ByteSize = dBit;				  // Number of bits/bytes, 4-8 
	PortDCB.Parity = NOPARITY;				  // 0-4=no,odd,even,mark,space 
	PortDCB.StopBits = ONESTOPBIT;			  // 0,1,2 = 1, 1.5, 2 

	// Configure the port according to the specifications of the DCB structure.
	if (!SetCommState(hPort, &PortDCB))
	{
		// Could not create the read thread.
		dwError = GetLastError();
		msg("Unable to configure the serial port");
		return EC_INVAL_CONFIG;
	}

	// Retrieve the time-out parameters for all read and write operations on the port. 
	GetCommTimeouts(hPort, &CommTimeouts);

	// Change the COMMTIMEOUTS structure settings.
	CommTimeouts.ReadIntervalTimeout = MAXDWORD;  
	CommTimeouts.ReadTotalTimeoutMultiplier = 0;  
	CommTimeouts.ReadTotalTimeoutConstant = 0;    
	CommTimeouts.WriteTotalTimeoutMultiplier = 10;  
	CommTimeouts.WriteTotalTimeoutConstant = 1000;    

	// Set the time-out parameters for all read and write operations on the port. 
	if (!SetCommTimeouts (hPort, &CommTimeouts))
	{
		// Could not set the port timeout parameter.
		dwError = GetLastError();
		msg("Unable to set the time-out parameters");
		return EC_TIMEOUT_SET;
	}

	return ecStatus;
}

