/************************************************************************************
 * NAME		: LabVIEWSerialDLL.cpp
 * DESC.	: DLL functions to interface to RS-232 serial port, called by LabVIEW.
 * PRGMER.	: Y. Bai
 * DATE		: 10/30/2003
 ************************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "LabVIEWSerialDLL.h"

DllExport long Setup(long comParms[])  //comParms[0] = port number, comParms[1] = baud_rate,
{								       //comParms[2] = data_bits,   comParms[3] = stop_bit
	ERR_CODE  rc = OK;			       //comParms[4] = parity,      comParms[5] = timeout_value
	pSerialCreate  pParam;
    fp = fopen("data.dat", "w");
	pParam = new SerialCreate;

	pParam->lBaudRate = comParms[1];
	pParam->lDataBits = comParms[2];
	pParam->lTimeout  = comParms[5];
	
	switch((int)comParms[0])
	{
	case 1:
		sPortName = "COM1";
		break;
	case 2:
		sPortName = "COM2";
		break;
	case 3:
		sPortName = "COM3";
		break;
	case 4:
		sPortName = "COM4";
		break;
	default:
		return EC_INVALIDPORT;
	}

	if (PortCreateflg)
	{
		msg("Port has been Setup ");
		return rc;
	}
	rc = PortInitialize(sPortName, pParam);
	if (rc != 0)
		msg("ERROR in PortInitialize()!");
	
	delete pParam;
	PortCreateflg = true;

	return rc;
}

DllExport UCHAR ReadByte(void)
{
	UCHAR ad_data;
	ERR_CODE rc = OK;
	CommPortClass* commClass;

	commClass = new CommPortClass;
	commClass->iMaxChars = 1;
	commClass->iMaxBits  = 8;
	
	rc = PortRead(commClass);
	if (rc != 0)
		msg("ERROR in PortRead()! ");

	fprintf(fp, "%d\n", commClass->pcBuffer);	//Reserve data
	ad_data = commClass->pcBuffer;
	delete commClass;

	return ad_data;
}

DllExport long Close()
{
	ERR_CODE rc = OK;

	if (PortCreateflg)
		CloseHandle(hPort);
	PortCreateflg = false;
    fclose(fp);
	return rc;
}

ERR_CODE PortInitialize(char* lpszPortName, pSerialCreate pCreate)
{
	DWORD  dwError;
	DCB	   PortDCB;  
	ERR_CODE       ecStatus = OK;
	COMMTIMEOUTS   CommTimeouts;
	UCHAR  dBit;

	// check if the port has been created...
	if (PortCreateflg)
	{
		msg("Port has been initialized!");
		return ecStatus;
	}
	// Open the serial port.
	hPort = CreateFile(lpszPortName,  // Pointer to the name of the port
					   GENERIC_READ | GENERIC_WRITE, // Access (read/write) mode
					   0,             // Share mode
					   NULL,          // Pointer to the security attribute
					   OPEN_EXISTING, // How to open the serial port
					   0,             // Port attributes
					   NULL);         // Handle to port with attribute to copy										

	// If it fails to open the port, return error.
	if ( hPort == INVALID_HANDLE_VALUE ) 
	{
		// Could not open the port.
		dwError = GetLastError();
		msg("Unable to open the port");
		CloseHandle(hPort);
		return EC_FOPEN;
	}
	  
	PortCreateflg = TRUE;
	PortDCB.DCBlength = sizeof(DCB);     

	// Get the default port setting information.
	GetCommState(hPort, &PortDCB);
	  
	// Change the DCB structure settings.
	PortDCB.BaudRate = pCreate->lBaudRate;    // Current baud 
	PortDCB.fBinary  = TRUE;				  // Binary mode; no EOF check 
	PortDCB.fParity  = FALSE;				  // Enable parity checking. 
	PortDCB.fOutxCtsFlow = FALSE;			  // No CTS output flow control 
	PortDCB.fOutxDsrFlow = FALSE;			  // No DSR output flow control 
	PortDCB.fDtrControl = DTR_CONTROL_ENABLE; // DTR flow control type 
	PortDCB.fDsrSensitivity = FALSE;		  // DSR sensitivity 
	PortDCB.fTXContinueOnXoff = FALSE;		  // FALSE	// XOFF continues Tx 
	PortDCB.fOutX = FALSE;					  // No XON/XOFF out flow control 
	PortDCB.fInX  = FALSE;					  // No XON/XOFF in flow control 
	PortDCB.fErrorChar = FALSE;				  // Disable error replacement. 
	PortDCB.fNull = FALSE;					  // Disable null stripping. 
	PortDCB.fRtsControl = RTS_CONTROL_ENABLE; // RTS flow control
	PortDCB.fAbortOnError = FALSE;			  // Do not abort reads/writes on error.
	dBit = (UCHAR)pCreate->lDataBits;		  // Assign data bits	
	PortDCB.ByteSize = dBit;				  // Number of bits/bytes, 4-8 
	PortDCB.Parity = NOPARITY;				  // 0-4=no,odd,even,mark,space 
	PortDCB.StopBits = ONESTOPBIT;			  // 0,1,2 = 1, 1.5, 2 

	// Configure the port according to the specifications of the DCB structure.
	if (!SetCommState(hPort, &PortDCB))
	{
		// Could not create the read thread.
		dwError = GetLastError();
		msg("Unable to configure the serial port");
		return EC_INVAL_CONFIG;
	}

	// Retrieve the time-out parameters for all read and write operations on the port. 
	GetCommTimeouts(hPort, &CommTimeouts);

	// Change the COMMTIMEOUTS structure settings.
	CommTimeouts.ReadIntervalTimeout = MAXDWORD;  
	CommTimeouts.ReadTotalTimeoutMultiplier = 0;  
	CommTimeouts.ReadTotalTimeoutConstant = 0;    
	CommTimeouts.WriteTotalTimeoutMultiplier = 10;  
	CommTimeouts.WriteTotalTimeoutConstant = 1000;    

	// Set the time-out parameters for all read and write operations on the port. 
	if (!SetCommTimeouts (hPort, &CommTimeouts))
	{
		// Could not set the port timeout parameter.
		dwError = GetLastError();
		msg("Unable to set the time-out parameters");
		return EC_TIMEOUT_SET;
	}

	EscapeCommFunction(hPort, SETDTR);	//Set DTR
	EscapeCommFunction(hPort, SETRTS);	//Set RTS
	ecStatus = PortWrite("0", 1);		//Output 0 to the TD line to make CLK LOW.
	if (ecStatus != 0)
		msg("ERROR in PortWrite()...Setup !");

	  return ecStatus;
}
/********************************************************************************
 * NAME		: PortWrite()
 * DESC.	: Write data to the port
 * DATE		: 11/9/03
 * PRGMER.	: Y. Bai
 ********************************************************************************/
ERR_CODE PortWrite(char* bByte, int NumByte)
{
	  DWORD dwError;
      DWORD dwNumBytesWritten;
	  ERR_CODE ecStatus = OK;

	  if (!WriteFile (hPort,				// Port handle
					  bByte,				// Pointer to the data to write 
					  NumByte,				// Number of bytes to write
					  &dwNumBytesWritten,	// Pointer to the number of bytes written
					  NULL))				// Must be NULL for Windows CE
	  {
		// WriteFile failed. Report error.
		dwError = GetLastError ();
		return EC_WRITE_FAIL;
	  }

	  return ecStatus;
}
/**********************************************************************************
 * NAME		: PortRead()
 * DESC.	: Read data from serial port
 * PGRMER.	: Y. Bai
 * DATE		: 10/31/03
 **********************************************************************************/
ERR_CODE PortRead(CommPortClass *hCommPort)
{  
	DWORD  dwModemStatus;
	int    index;
	UCHAR  ad_data = 0;
	CommPortClass*  CommPorts;
	ERR_CODE ecStatus = OK;

	CommPorts = (CommPortClass* )hCommPort;
	
	EscapeCommFunction(hPort, CLRRTS);		// Reset RTS to LOW to enable the CS-bar  
	for (index = 0; index < CommPorts->iMaxBits; index++)
	{
		ad_data = ad_data << 1;
		ecStatus = PortWrite("255", 1);		//Output 1 to the TD line to make CLK HIGH.
		if (ecStatus != 0)
		{
			msg("ERROR in PortWrite()...HIGH !");
			return ecStatus;
		}
		if (!GetCommModemStatus(hPort, &dwModemStatus))
		{
			msg("ERROR in GetModemStatus()!");
			return EC_GETMODSTATUS;
		}
		if (MS_CTS_ON & dwModemStatus)		// Get the CTS status.
			ad_data++;
		ecStatus = PortWrite("0", 1);		//Output 1 to the TD line to make CLK HIGH.
		if (ecStatus != 0)
		{
			msg("ERROR in PortWrite()...LOW !");
			return ecStatus;
		}
	}
	EscapeCommFunction(hPort, SETRTS);		// Set RTS to HIGH to disable the CS-bar  
	CommPorts->pcBuffer = ad_data;

	return ecStatus;
}
	
