/***********************************************************************************
 * NAME		: MAXDLL.cpp
 * DESC.	: Source file for MAXDLL.DLL. Called by MATLAB loadlibrary func.
 * PGRMER.	: Y. Bai
 * DATE		: 11/26/2003
 **********************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include "MAXDLL.h"

#define   DEBUG

DllExport int Setup(int* pCom)
{
	ERR_CODE  rc = OK;					//pCom[1] = baud_rate   
	pSerialCreate  pParam;			    //pCom[2] = data_bits
	fp = fopen("max_dll.dat", "w");		//pCom[3] = stop_bit 
	pParam = new SerialCreate;			//pCom[4] = parity
										//pCom[5] = timeout_value
	pParam->lBaudRate = pCom[1];
	pParam->lDataBits = pCom[2];
	pParam->lTimeout  = pCom[5];
	
	#ifdef DEBUG
	mexPrintf("data[%d] = %d\n", 0, pCom[0]);
	mexPrintf("data[%d] = %d\n", 1, pCom[1]);
	mexPrintf("data[%d] = %d\n", 2, pCom[2]);
	mexPrintf("data[%d] = %d\n", 3, pCom[3]);
	mexPrintf("data[%d] = %d\n", 4, pCom[4]);
	mexPrintf("data[%d] = %d\n", 5, pCom[5]);
	#endif

	switch(pCom[0])
	{
	case 1: sPortName = "COM1";
			break;
	case 2: sPortName = "COM2";
			break;
	case 3: sPortName = "COM3";
			break;
	case 4: sPortName = "COM4";
			break;
	default:return EC_INVALIDPORT;
	}

	if (PortCreateflg)
	{
		msg("Port has been Setup ");
		return rc;
	}
	rc = PortInitialize(sPortName, pParam);
	if (rc != 0)
		msg("ERROR in PortInitialize()!");
	#ifdef DEBUG
	else
		mexPrintf("\nPort is initialized...\n");
	#endif

	delete pParam;
	PortCreateflg = true;

	return rc;
}
DllExport int ReadData()
{
	int ad_data;
	ERR_CODE rc = OK;
	CommPortClass* commClass;

	commClass = new CommPortClass;
	commClass->iMaxChars = MAX_BYTE;
	commClass->iMaxBits  = MAX_BITS;
	
	EscapeCommFunction(hPort, SETDTR);	//Set DTR to disable the SCLK
	EscapeCommFunction(hPort, CLRRTS);	//Reset RTS to disable the CS-bar
 
	rc = PortRead(commClass);
	if (rc != 0 && runTIMES != 0)		//1st read error neglected.
		msg("ERROR in PortRead()! ");
	
	runTIMES++;
	fprintf(fp, "%d\n", commClass->pcBuffer);	//Reserve data
	ad_data = commClass->pcBuffer;
	delete commClass;

	return ad_data;
}

DllExport int PortClose()
{
	ERR_CODE rc = OK;

	if (PortCreateflg)
		CloseHandle(hPort);
	PortCreateflg = false;
    fclose(fp);
	runTIMES = 0;
    #ifdef DEBUG
		mexPrintf("\nPort is closed...\n");
	#endif

	return rc;
}

ERR_CODE PortInitialize(char* lpszPortName, pSerialCreate pCreate)
{
	UCHAR  dBit;
	DWORD  dwError;
	DCB	   PortDCB;  
	ERR_CODE       ecStatus = OK;
	COMMTIMEOUTS   CommTimeouts;

	// check if the port has been created...
	if (PortCreateflg)
	{
		msg("Port has been initialized!");
		return ecStatus;
	}
	// Open the serial port.
	hPort = CreateFile(lpszPortName,  // Pointer to the name of the port
					   GENERIC_READ | GENERIC_WRITE, // Access (read/write) mode
					   0,             // Share mode
					   NULL,          // Pointer to the security attribute
					   OPEN_EXISTING, // How to open the serial port
					   0,             // Port attributes
					   NULL);         // Handle to port with attribute to copy										

	// If it fails to open the port, return error.
	if ( hPort == INVALID_HANDLE_VALUE ) 
	{
		// Could not open the port.
		dwError = GetLastError();
		msg("Unable to open the port");
		CloseHandle(hPort);
		return EC_FOPEN;
	} 
	PortCreateflg = TRUE;
	PortDCB.DCBlength = sizeof(DCB);     

	// Get the default port setting information.
	GetCommState(hPort, &PortDCB);
	  
	// Change the DCB structure settings.
	PortDCB.BaudRate = pCreate->lBaudRate;    // Current baud 
	PortDCB.fBinary  = TRUE;				  // Binary mode; no EOF check 
	PortDCB.fParity  = FALSE;				  // Enable parity checking. 
	PortDCB.fOutxCtsFlow = FALSE;			  // No CTS output flow control 
	PortDCB.fOutxDsrFlow = FALSE;			  // No DSR output flow control 
	PortDCB.fDtrControl = DTR_CONTROL_ENABLE; // DTR flow control type 
	PortDCB.fDsrSensitivity = FALSE;		  // DSR sensitivity 
	PortDCB.fTXContinueOnXoff = FALSE;		  // FALSE	// XOFF continues Tx 
	PortDCB.fOutX = FALSE;					  // No XON/XOFF out flow control 
	PortDCB.fInX  = FALSE;					  // No XON/XOFF in flow control 
	PortDCB.fErrorChar = FALSE;				  // Disable error replacement. 
	PortDCB.fNull = FALSE;					  // Disable null stripping. 
	PortDCB.fRtsControl = RTS_CONTROL_ENABLE; // RTS flow control
	PortDCB.fAbortOnError = FALSE;			  // Do not abort reads/writes on error.
	dBit = (UCHAR)pCreate->lDataBits;		  // Assign data bits	
	PortDCB.ByteSize = dBit;				  // Number of bits/bytes, 4-8 
	PortDCB.Parity = NOPARITY;				  // 0-4=no,odd,even,mark,space 
	PortDCB.StopBits = ONESTOPBIT;			  // 0,1,2 = 1, 1.5, 2 

	// Configure the port according to the specifications of the DCB structure.
	if (!SetCommState(hPort, &PortDCB))
	{
		// Could not create the read thread.
		dwError = GetLastError();
		msg("Unable to configure the serial port");
		return EC_INVAL_CONFIG;
	}

	// Retrieve the time-out parameters for all read and write operations on the port. 
	GetCommTimeouts(hPort, &CommTimeouts);

	// Change the COMMTIMEOUTS structure settings.
	CommTimeouts.ReadIntervalTimeout = MAXDWORD;  
	CommTimeouts.ReadTotalTimeoutMultiplier = 0;  
	CommTimeouts.ReadTotalTimeoutConstant = 0;    
	CommTimeouts.WriteTotalTimeoutMultiplier = 10;  
	CommTimeouts.WriteTotalTimeoutConstant = 1000;    

	// Set the time-out parameters for all read and write operations on the port. 
	if (!SetCommTimeouts (hPort, &CommTimeouts))
	{
		// Could not set the port timeout parameter.
		dwError = GetLastError();
		msg("Unable to set the time-out parameters");
		return EC_TIMEOUT_SET;
	}

	return ecStatus;
}
/**********************************************************************************
 * NAME		: PortRead()
 * DESC.	: Read data from serial port
 * PGRMER.	: Y. Bai
 * DATE		: 11/21/03
 **********************************************************************************/
ERR_CODE PortRead(CommPortClass *hCommPort)
{  
	HANDLE hThread;			// handler for port read thread
	DWORD  IDThread;
	DWORD  Ret, ExitCode;
	DWORD  dTimeout = 500;	// define time out value: 500 ms.
	ERR_CODE ecStatus = OK;

	if (!(hThread = CreateThread(NULL,			    // no security attributes
				    0,					            // use default stack size
				    (LPTHREAD_START_ROUTINE) ThreadFunc,
				    (LPVOID)hCommPort,              // parameter to thread function
				    CREATE_SUSPENDED,	            // creation flag - suspended
				    &IDThread) ) )					// returns thread ID
    {
		msg("Create Read Thread failed");
	 	return EC_CREATE_THREAD;
	}

	ResumeThread(hThread);		// start thread now

	Ret = WaitForSingleObject(hThread, dTimeout);
	if (Ret == WAIT_OBJECT_0)
	{
		// data received & process it...need to do nothing because the data 
		// has been stored in the hCommPort in Thread Func. Close thread handle
		CloseHandle(hThread);
	}
	else if (Ret == WAIT_TIMEOUT)
	{
		// time out happened, warning & kill thread
		Ret = GetExitCodeThread(hThread, &ExitCode);
		if (ExitCode == STILL_ACTIVE)
		{
			TerminateThread(hThread, ExitCode);
			CloseHandle(hThread);
			return EC_RECV_TIMEOUT;
		}
		else
		{
			CloseHandle(hThread);
			msg("ERROR in GetExitCodeThread: != STILL_ACTIVE ");
			ecStatus = EC_EXIT_CODE;
		}
	}
	else
	{
		msg("ERROR in WaitFor SingleObject ");
		ecStatus = EC_WAIT_SINGLEOBJ;
	}
	  
	return ecStatus;
}
/***************************************************************************************
* NAME		: ThreadFunc()
* DESC.		: This funciton starts to read the data from the port.
*			: The reason to use this thread is to overcome the hung up introduced by the
*			: WaitCommEvent() function which waits for only the comm. event, not synchro-
*			: nition event. The program will be dead-cycle without this thread.
* PGRMER.	: Y. Bai
* DATE		: 11/26/03
****************************************************************************************/
void WINAPI ThreadFunc(void* hCommPorts)
{
	BOOL   fDone;
	int    ad_data = 0;
	DWORD  dwModemStatus;
	int    index, nTotRead = 0;
 	DWORD  dwCommModemStatus;  
	CommPortClass*  CommPorts;
	ERR_CODE ecStatus = OK;

	CommPorts = (CommPortClass* )hCommPorts;	

	EscapeCommFunction(hPort, SETRTS);		// Set RTS to enable the CS-bar to start A/D 
	// Specify a set of events to be monitored for the port.
 	SetCommMask(hPort, EV_DSR);
	fDone = FALSE;
 	while (!fDone) 
 	{
		// Wait for an event to occur for the port.
 		WaitCommEvent(hPort, &dwCommModemStatus, 0);
 		if (dwCommModemStatus & EV_DSR)	// received the DSR event
 		{
 		    for (index = 0; index < CommPorts->iMaxBits; index++)
			{
				EscapeCommFunction(hPort, CLRDTR);	//Reset DTR (provide HIGH in SCLK) 
				Delay(10);
				EscapeCommFunction(hPort, SETDTR);	//Set DTR (provide LOW in SCLK) 
				Delay(10);		
				if (!GetCommModemStatus(hPort, &dwModemStatus))
				{
					msg("ERROR in GetModemStatus()!");
					fDone = TRUE;
					break;
				}
				if (MS_DSR_ON & dwModemStatus)		//DSR status is HIGH.
					ad_data = ad_data + 0;
				else								//DSR status is LOW
					ad_data++;
				ad_data = ad_data << 1;
			}
			EscapeCommFunction(hPort, SETDTR);	//Set DTR (provide LOW in SCLK) 
			EscapeCommFunction(hPort, CLRRTS);	//Reset CS to deselect A/D 
			ad_data = ad_data >> 1;
			CommPorts->pcBuffer = ad_data;
			fDone = TRUE;
			break;
		}
	}
}
void Delay(int num)
{
	int l, m, n = 100*num;

	for (m = 0; m < n; m++)
		l++;
}
