/***********************************************************************************
 * NAME		: MEX_TLC.cpp
 * DESC.	: Source file for MEX-File MEX_TLC.DLL. Called by MATLAB mextlc.m
 * PGRMER.	: Y. Bai
 * DATE		: 11/20/2003
 **********************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include "MEX_TLC.h"

#define   DEBUG

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]) 
{
	int ad_data[100];
	int i, m, n, ret = 0;
	double *inParam, *outParam, *plpr;
	
	m = 1; n = 100;			  // define the output array dimension			
	// Get the data passed in 
	for (i = 0; i < nrhs; i++)		
		inParam = mxGetPr(prhs[i]);	

	ret = Setup(inParam);
	if (ret != OK)
	    mexErrMsgTxt("ERROR in Setup() calling!"); 
		
	outParam = (double *) mxMalloc(m*n * sizeof(double));
    plhs[0] = mxCreateDoubleMatrix(m, n, mxREAL);
	plpr = mxGetPr(plhs[0]);

	#ifdef DEBUG
	mexPrintf("Port is reading...\n");
	#endif
	for (i = 0; i < MAXBYTES; i++)
	{
		ad_data[i]  = (int)ReadByte();
		outParam[i] = (double)ad_data[i];
		*(plpr + i) = outParam[i];
	}	
	ret = PortClose();
	#ifdef DEBUG
	mexPrintf("Port is closed...\n");
	#endif

	return;
}

int Setup(double* pCom)
{
	ERR_CODE  rc = OK;					//pCom[1] = baud_rate   
	pSerialCreate  pParam;			    //pCom[2] = data_bits
	fp = fopen("data.dat", "w");		//pCom[3] = stop_bit 
	pParam = new SerialCreate;			//pCom[4] = parity
										//pCom[5] = timeout_value
	pParam->lBaudRate = (int)pCom[1];
	pParam->lDataBits = (int)pCom[2];
	pParam->lTimeout  = (int)pCom[5];
	
	#ifdef DEBUG
	mexPrintf("data[%d] = %d\n", 0, (int)pCom[0]);
	mexPrintf("data[%d] = %d\n", 1, (int)pCom[1]);
	mexPrintf("data[%d] = %d\n", 2, (int)pCom[2]);
	mexPrintf("data[%d] = %d\n", 3, (int)pCom[3]);
	mexPrintf("data[%d] = %d\n", 4, (int)pCom[4]);
	mexPrintf("data[%d] = %d\n", 5, (int)pCom[5]);
	#endif

	switch((int)pCom[0])
	{
	case 1:
		sPortName = "COM1";
		break;
	case 2:
		sPortName = "COM2";
		break;
	case 3:
		sPortName = "COM3";
		break;
	case 4:
		sPortName = "COM4";
		break;
	default:
		return EC_INVALIDPORT;
	}

	if (PortCreateflg)
	{
		msg("Port has been Setup ");
		return rc;
	}
	rc = PortInitialize(sPortName, pParam);
	if (rc != 0)
		msg("ERROR in PortInitialize()!");
	#ifdef DEBUG
	else
		mexPrintf("\nPort is initialized...\n");
	#endif

	delete pParam;
	PortCreateflg = true;

	return rc;
}
UCHAR ReadByte(void)
{
	UCHAR ad_data;
	ERR_CODE rc = OK;
	CommPortClass* commClass;

	commClass = new CommPortClass;
	commClass->iMaxChars = 1;
	commClass->iMaxBits  = 8;
	
	rc = PortRead(commClass);
	if (rc != 0)
		msg("ERROR in PortRead()! ");

	fprintf(fp, "%d\n", commClass->pcBuffer);	//Reserve data
	ad_data = commClass->pcBuffer;
	delete commClass;

	return ad_data;
}

int PortClose()
{
	ERR_CODE rc = OK;

	if (PortCreateflg)
		CloseHandle(hPort);
	PortCreateflg = false;
    fclose(fp);
	return rc;
}

ERR_CODE PortInitialize(char* lpszPortName, pSerialCreate pCreate)
{
	UCHAR  dBit;
	DWORD  dwError;
	DCB	   PortDCB;  
	ERR_CODE       ecStatus = OK;
	COMMTIMEOUTS   CommTimeouts;

	// check if the port has been created...
	if (PortCreateflg)
	{
		msg("Port has been initialized!");
		return ecStatus;
	}
	// Open the serial port.
	hPort = CreateFile(lpszPortName,  // Pointer to the name of the port
					   GENERIC_READ | GENERIC_WRITE, // Access (read/write) mode
					   0,             // Share mode
					   NULL,          // Pointer to the security attribute
					   OPEN_EXISTING, // How to open the serial port
					   0,             // Port attributes
					   NULL);         // Handle to port with attribute to copy										

	// If it fails to open the port, return error.
	if ( hPort == INVALID_HANDLE_VALUE ) 
	{
		// Could not open the port.
		dwError = GetLastError();
		msg("Unable to open the port");
		CloseHandle(hPort);
		return EC_FOPEN;
	} 
	PortCreateflg = TRUE;
	PortDCB.DCBlength = sizeof(DCB);     

	// Get the default port setting information.
	GetCommState(hPort, &PortDCB);
	  
	// Change the DCB structure settings.
	PortDCB.BaudRate = pCreate->lBaudRate;    // Current baud 
	PortDCB.fBinary  = TRUE;				  // Binary mode; no EOF check 
	PortDCB.fParity  = FALSE;				  // Enable parity checking. 
	PortDCB.fOutxCtsFlow = FALSE;			  // No CTS output flow control 
	PortDCB.fOutxDsrFlow = FALSE;			  // No DSR output flow control 
	PortDCB.fDtrControl = DTR_CONTROL_ENABLE; // DTR flow control type 
	PortDCB.fDsrSensitivity = FALSE;		  // DSR sensitivity 
	PortDCB.fTXContinueOnXoff = FALSE;		  // FALSE	// XOFF continues Tx 
	PortDCB.fOutX = FALSE;					  // No XON/XOFF out flow control 
	PortDCB.fInX  = FALSE;					  // No XON/XOFF in flow control 
	PortDCB.fErrorChar = FALSE;				  // Disable error replacement. 
	PortDCB.fNull = FALSE;					  // Disable null stripping. 
	PortDCB.fRtsControl = RTS_CONTROL_ENABLE; // RTS flow control
	PortDCB.fAbortOnError = FALSE;			  // Do not abort reads/writes on error.
	dBit = (UCHAR)pCreate->lDataBits;		  // Assign data bits	
	PortDCB.ByteSize = dBit;				  // Number of bits/bytes, 4-8 
	PortDCB.Parity = NOPARITY;				  // 0-4=no,odd,even,mark,space 
	PortDCB.StopBits = ONESTOPBIT;			  // 0,1,2 = 1, 1.5, 2 

	// Configure the port according to the specifications of the DCB structure.
	if (!SetCommState(hPort, &PortDCB))
	{
		// Could not create the read thread.
		dwError = GetLastError();
		msg("Unable to configure the serial port");
		return EC_INVAL_CONFIG;
	}

	// Retrieve the time-out parameters for all read and write operations on the port. 
	GetCommTimeouts(hPort, &CommTimeouts);

	// Change the COMMTIMEOUTS structure settings.
	CommTimeouts.ReadIntervalTimeout = MAXDWORD;  
	CommTimeouts.ReadTotalTimeoutMultiplier = 0;  
	CommTimeouts.ReadTotalTimeoutConstant = 0;    
	CommTimeouts.WriteTotalTimeoutMultiplier = 10;  
	CommTimeouts.WriteTotalTimeoutConstant = 1000;    

	// Set the time-out parameters for all read and write operations on the port. 
	if (!SetCommTimeouts (hPort, &CommTimeouts))
	{
		// Could not set the port timeout parameter.
		dwError = GetLastError();
		msg("Unable to set the time-out parameters");
		return EC_TIMEOUT_SET;
	}

	EscapeCommFunction(hPort, SETDTR);	//Set DTR
	EscapeCommFunction(hPort, SETRTS);	//Set RTS
	ecStatus = PortWrite("0", 1);		//Output 0 to the TD line to make CLK LOW.
	if (ecStatus != 0)
		msg("ERROR in PortWrite()...Setup !");

	  return ecStatus;
}
/********************************************************************************
 * NAME		: PortWrite()
 * DESC.	: Write data to the port
 * DATE		: 11/9/03
 * PRGMER.	: Y. Bai
 ********************************************************************************/
ERR_CODE PortWrite(char* bByte, int NumByte)
{
	  DWORD dwError;
      DWORD dwNumBytesWritten;
	  ERR_CODE ecStatus = OK;

	  if (!WriteFile (hPort,				// Port handle
					  bByte,				// Pointer to the data to write 
					  NumByte,				// Number of bytes to write
					  &dwNumBytesWritten,	// Pointer to the number of bytes written
					  NULL))				// Must be NULL for Windows CE
	  {
		// WriteFile failed. Report error.
		dwError = GetLastError ();
		return EC_WRITE_FAIL;
	  }

	  return ecStatus;
}
/**********************************************************************************
 * NAME		: PortRead()
 * DESC.	: Read data from serial port
 * PGRMER.	: Y. Bai
 * DATE		: 10/31/03
 **********************************************************************************/
ERR_CODE PortRead(CommPortClass *hCommPort)
{  
	DWORD  dwModemStatus;
	int    index;
	UCHAR  ad_data = 0;
	CommPortClass*  CommPorts;
	ERR_CODE ecStatus = OK;

	CommPorts = (CommPortClass* )hCommPort;
	
	EscapeCommFunction(hPort, CLRRTS);		// Reset RTS to LOW to enable the CS-bar  
	for (index = 0; index < CommPorts->iMaxBits; index++)
	{
		ad_data = ad_data << 1;
		ecStatus = PortWrite("255", 1);		//Output 1 to the TD line to make CLK HIGH.
		if (ecStatus != 0)
		{
			msg("ERROR in PortWrite()...HIGH !");
			return ecStatus;
		}
		if (!GetCommModemStatus(hPort, &dwModemStatus))
		{
			msg("ERROR in GetModemStatus()!");
			return EC_GETMODSTATUS;
		}
		if (MS_CTS_ON & dwModemStatus)		// Get the CTS status.
			ad_data++;
		ecStatus = PortWrite("0", 1);		//Output 1 to the TD line to make CLK HIGH.
		if (ecStatus != 0)
		{
			msg("ERROR in PortWrite()...LOW !");
			return ecStatus;
		}
	}
	EscapeCommFunction(hPort, SETRTS);		// Set RTS to HIGH to disable the CS-bar  
	CommPorts->pcBuffer = ad_data;

	return ecStatus;
}
	
